package simulation;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import commandSystem.Direction;
import commandSystem.ElevatorListener;


public class Elevator {
	static final int PRECISION = 100;
	private double floorHeight;
	private double step;
	private double slow_step;
	private double height;
	private Direction direction = Direction.NONE;
	private boolean stoppingNextFloor = false;
	private boolean emergency = false;
	private ElevatorListener listener;
	private int nbFloors;

	public Elevator(int currentFloor, int nbFloors) {
		this.nbFloors = nbFloors;
		height = (double) currentFloor / (nbFloors - 1); // -1 car rez-de-chaussée
		floorHeight = 1. / (nbFloors - 1);
		step = floorHeight / PRECISION;
		slow_step = step / 2;
		System.out.println("step: " + step + ", floorHeight: " + floorHeight);
	}

	public void setActionListener(ElevatorListener listener) {
		this.listener = listener;
	}

	public void goUp() {
		if (emergency) return;
		stoppingNextFloor = false;
		direction = Direction.UP;
	}

	public void goDown() {
		if (emergency) return;
		stoppingNextFloor = false;
		direction = Direction.DOWN;
	}

	public void stop() {
		direction = Direction.NONE;
		stoppingNextFloor = false;
	}

	public void stopNextFloor() {
		stoppingNextFloor = true;
	}

	public void emergencyStop() {
		emergency = true;
		direction = Direction.NONE;
	}

	public void cancelEmergency() {
		emergency = false;
	}

	public void update() {
		if (emergency) return;

		double delta;
		if (stoppingNextFloor) {
			delta = slow_step;
		}
		else {
			delta = step;
		}

		switch (direction) {
		case DOWN:
			height -= delta;
			if (height % floorHeight < delta && listener != null) {
				listener.floorChange(Direction.DOWN);
			}
			break;
		case UP:
			height += delta;
			if (height % floorHeight < delta && listener != null) {
				listener.floorChange(Direction.UP);
			}
			break;
		default:
			break;
		}
	}

	public Direction getDirection() {
		return direction;
	}

	public boolean getStoppingNextFloor() {
		return stoppingNextFloor;
	}

	public double getHeight() {
		return height;
	}

	public boolean getEmergency() {
		return emergency;
	}

	public int getNbFloors() {
		return nbFloors;
	}
}
